<?php
// $Id: autoadmin.schema.inc,v 1.1 2009/09/18 20:44:36 rune Exp $

/**
 * @file
 * Contains functions for interfacing with schemas
 *
 * @ingroup autoadmin
 * @author Rune Kaagaard rune@prescriba.com
 */

// Constants.
global $language;
define('AUTOADMIN_CACHE_NAME', 'autoadmin' . $language->language);

/**
 * Rebuilds all autoadmin cache.
 *
 * @return array empty array
 */
function autoadmin_scema_rebuild_cache() {
  $schemas = array();
  $schemas = _autoadmin_schemas_load_schemas();
  foreach ($schemas as $table => $schema) {
    if (autoadmin_schema_has_autoadmin($schema)) {
      $schemas[$table] = _autoadmin_schema_process_schema($schema, $table);
      // Dont use autoadmin schemas with errors.
      if ($schemas[$table]['has_error'] === TRUE) {
        unset($schemas[$table]);
      }
    }
  }
  cache_set(AUTOADMIN_CACHE_NAME, $schemas);
  variable_set('autoadmin_rebuild_menu_needed', TRUE);
  return array();
}

/**
 * retrive schema from cache
 *
 * @param string $table
 * @return array/FALSE
 */
function autoadmin_schema_get_schema($table) {
  static $cache;
  // First time load cache.
  if (empty($cache->data)) {
    $cache = cache_get(AUTOADMIN_CACHE_NAME);
  }
  // If still empty, the cache has been cleared and rebuild is needed.
  if (empty($cache->data)) {
    autoadmin_scema_rebuild_cache();
    $cache = cache_get(AUTOADMIN_CACHE_NAME);
    if (empty($cache->data)) {
      return autoadmin_add_error(t('Could not rebuild cache'));
    }
  }
  // If table is NULL return all cache.
  if (!$table) {
    return $cache->data;
  }
  // Return schema for single table.
  if (!empty($cache->data[$table])) {
    return $cache->data[$table];
  }
  else {
    return autoadmin_add_error(t('Table %table not found', array('%table' => $table)));
  }
}

/**
 * Loads schemas from .install files the hook_autoadmin() hooks and returns a merged array.
 *
 * @return array
 */
function _autoadmin_schemas_load_schemas() {
  $modules = module_list();
  $install_schemas = array();
  foreach ($modules as $module) {
    $module_schemas = drupal_get_schema_unprocessed($module);
    if (!empty($module_schemas)) {
      $install_schemas = array_merge($install_schemas, $module_schemas);
    }
  }
  $autoadmin_schemas = module_invoke_all('autoadmin_schema');
  return autoadmin_util_array_merge_recursive_distinct($install_schemas, $autoadmin_schemas);
}

/**
 * Process and validate schema.
 *
 * @param array $schema
 * @param string $table if this is set only prepare one schema, otherwise all schemas
 * @return array
 */
function _autoadmin_schema_process_schema(&$schema, $table) {
  _autoadmin_schema_default_schema($schema, $table);
  $schema = _autoadmin_schema_process_fields($schema);
  $schema = _autoadmin_schema_process_relations($schema);
  // Which argument num to use as primary value.
  $schema['argument_num'] = substr_count($schema['path'], '/') + 1;
  _autoadmin_schema_validate_schema($schema);
  return $schema;
}

/**
 * Sets default values for schema.
 * 
 * @param <type> $schema
 *   By ref
 * @param string $table
 */
function _autoadmin_schema_default_schema(&$schema, $table) {
  $schema = array_merge(array(
      'table' => $table,
    ),
    $schema
  );
}

/**
 * Set default values for all fields in schema.
 *
 * @param schema $schema
 * @return schema
 */
function _autoadmin_schema_process_fields($schema) {
  foreach ($schema['fields'] as $field_key => &$field) {
    _autoadmin_schema_process_field($schema, $field_key, $field);
  }
  return $schema;
}

/**
 * Adds relation to the field itself and grouped by relation type.
 *
 * @param array $schema changed by ref
 * @param array $field changed by ref
 */
function _autoadmin_schema_process_relations($schema) {
  if (empty($schema['relations'])) {
    return $schema;
  }
  foreach ($schema['relations'] as $foreign_table => &$relation) {
    if (empty($relation['local_key'])) {
       return $schema;
    }
    $local_key = $relation['local_key'];
    if (!empty($schema['fields'][$local_key])) {
      $relation['foreign_table'] = $foreign_table;
      $type = $relation['type'];
      // Hook up references
      $schema['fields'][$local_key]['relation'] = &$relation;
      $schema['relations_by_type'][$type][$local_key] = &$relation;
      // Default relation settings.
      _autoadmin_schema_default_relation($relation);
    }
  }
  return $schema;
}

/**
 * add default relation settings to the field
 *
 * @param array $relation changed by ref
 */
function _autoadmin_schema_default_relation(&$relation) {
  $relation = array_merge(array(
      'cascading_delete' => FALSE,
    ),
    $relation
  );
}

/**
 * Set default value for a single field.
 *
 * @param array $schema changed by reference
 * @param string $field_key
 * @param array $field changed by reference
 */
function _autoadmin_schema_process_field($schema, $field_key, &$field) {
  $field['key'] = $field_key;
  $field['autoadmin_type'] = _autoadmin_schema_get_autoadmin_type($schema, $field);
}

/**
 * Returns the autoadmin type for a field.
 *
 * @param array $schema
 * @param array $field
 *  By ref.
 * @return NULL
 */
function _autoadmin_schema_get_autoadmin_type($schema, &$field) {
  // Change per name of field.
  switch ($field['key']) {
    case 'weight':
      return 'weight';
  }
  // Change if primary.
  if ($field['key'] == $schema['primary key'][0]) {
    return 'primary_key';
  }
  // Change into bool if its a tiny int.
  if ($field['type'] == 'int' && !empty($field['size']) && $field['size'] == 'tiny') {
    return 'bool';
  }
  // Default to field type.
  return $field['type'];
}

/**
 * Returns the a schemas primary field
 * @param array $schema
 * @return array
 */
function autoadmin_schema_get_primary_field($schema) {
  $primary_field = $schema['primary key'][0];
  return $schema['fields'][$primary_field];
}

/**
 * Check if schema has autoadmin settings
 * 
 * @param array $schema
 * @return bool
 */
function autoadmin_schema_has_autoadmin($schema) {
  return !empty($schema['path']);
}

/**
 * Validates schema, send errors messages and marks schema as having error.
 *
 * @todo could be split into more functions
 * @param array $schema
 */
function _autoadmin_schema_validate_schema(&$schema) {
  if (empty($schema)) {
    return autoadmin_add_error(t('Schemas are empty'));
  }
  $schema['has_error'] = FALSE;
  $required_main_settings = array('title', 'path', 'alias', 'title_plural');
  foreach ($required_main_settings as $setting) {
    if (empty($schema[$setting])) {
      autoadmin_add_error(
        t('Schema must have setting: !setting', array('!setting' => $setting)), $schema)
      ;
    }
  }
  // The field types currently supported by autoadmin.
    $supported_field_types = array(
      'int', 'varchar', 'text', 'serial', 'has_one', 'has_many', 'primary_key', 'weight', 'bool'
  );
  // The relations types currently supported by autoadmin.
  $required_relation_settings = array(
    'type' => array('has_many', 'has_one'),
    'foreign_key' => NULL,
    'foreign_alias' => NULL,
    'foreign_table' => NULL,
  );
  $table = $schema['table'];
  if (empty($schema['fields'])) {
    autoadmin_add_error(t('No fields found'), $schema);
  }
  // Check fields.
  foreach ($schema['fields'] as $field_key => $field) {
    if (empty($field['type'])) {
      autoadmin_add_error(t('Field must have a type'), $schema, $field);
    }
    elseif (!in_array($field['type'], $supported_field_types)) {
      autoadmin_add_error(t('Type !field_type is not supported by autoadmin', array('!field_type' => $field['type'])), $schema, $field);
    }
  }

  if (empty($schema['relations'])) {
    return;
  }
  // Check if relations are correct.
  foreach ($schema['relations'] as $field_key => $relation) {
    $relation_keys = array_keys($relation);
    foreach ($required_relation_settings as $required_key => $required_value) {
      // Check keys.
      if (!in_array($required_key, $relation_keys)) {
        autoadmin_add_error(
          t('Relation parameters must contain setting "!key"',
            array('!key' => $required_key)
          ),
          $schema,
          $field
      );
      // Check values.
      }
      elseif ($required_value && !in_array($relation[$required_key], $required_value)) {
        autoadmin_add_error(
          t('Value for "!key" must be either "!values". Current value is "!value"',
            array('!field' => $field_key, '!key' => $required_key, '!values' => implode(' or ', $required_value), '!value' => $relation[$required_key])
          ),
          $schema,
          $schema['fields'][$field_key]
        );
      }
    }
  }
}